"""
arcgis-sdk
~~~~~~~~~~
Python SDK for Arcgis API
:copyright: (c) 2017 mongkok
:license: MIT, see LICENSE for more details
"""

import requests


__version__ = '0.0.2'

ARCGIS_API_URL = 'https://www.arcgis.com/sharing/rest/'


class ArcgisAPI(object):
    """
    Docs: https://resources.arcgis.com/en/help/arcgis-rest-api/
    """

    def __init__(self, access_token, timeout=None,
                 proxies=None, session=None):

        self.access_token = access_token
        self.timeout = timeout
        self.proxies = proxies
        self.session = session or requests.Session()

    def refresh_token(self, client_id, refresh_token):
        result = self.request('oauth2/token/', {
            'client_id': client_id,
            'refresh_token': refresh_token,
            'grant_type': 'refresh_token'
        })

        self.access_token = result.get('access_token')
        return result

    def self(self):
        return self.request('portals/self')

    def users(self, **params):
        return self.request(
            'community/users',
            params=params)

    def user_detail(self, username):
        return self.request(
            "community/users/{username}".format(username=username))

    def groups(self, **params):
        return self.request(
            'community/groups',
            params=params)

    def create_group(self, **data):
        return self.post(
            'community/createGroup',
            data=data)

    def update_group(self, group_id, **data):
        return self.post(
            'community/groups/{group_id}/update'.format(group_id=group_id),
            data=data)

    def delete_group(self, group_id):
        return self.post(
            'community/groups/{group_id}/delete'.format(group_id=group_id))

    def add_users_to_group(self, group_id, users):
        return self.post(
            'community/groups/{group_id}/addUsers'.format(group_id=group_id),
            data={'users': users})

    def invite_to_group(self, group_id, **data):
        return self.post(
            'community/groups/{group_id}/invite'.format(group_id=group_id),
            data=data)

    def add_item(self, username, **data):
        return self.post(
            'content/users/{username}/addItem'.format(username=username),
            data=data)

    def post(self, *args, **kwargs):
        return self.request(method='POST', *args, **kwargs)

    def request(self, path, params=None, data=None, files=None, method=None):
        if params is None:
            params = dict()

        if params.get('grant_type') != 'refresh_token':
            params.update({
                'token': self.access_token,
                'f': 'json'
            })

        response = self.session.request(
            method or 'GET',
            ARCGIS_API_URL + path,
            data=data,
            params=params,
            proxies=self.proxies,
            timeout=self.timeout,
            files=files)

        if not 200 <= response.status_code < 300:
            raise ArcgisAPIError(code=response.status_code)

        result = response.json()

        if result.get('error'):
            raise ArcgisAPIError(result)
        return result


class ArcgisAPIError(Exception):

    def __init__(self, result=None, code=None):
        self.result = result

        if isinstance(result, dict):
            self.code = result['error']['code']
            self.details = result['error']['details']
            self.message = result['error']['message']
        else:
            self.code = code
            self.message = 'Server error'

        Exception.__init__(self, self.message)



import os
import re

from setuptools import find_packages
from setuptools import setup


def get_long_description():
    with open('README.rst', 'r') as f:
        return f.read()


def get_version(package):
    with open(os.path.join(package, '__init__.py')) as f:
        pattern = r'^__version__ = [\'"]([^\'"]*)[\'"]'
        return re.search(pattern, f.read(), re.MULTILINE).group(1)


setup(
    name='arcgis-sdk',
    version=get_version('arcgis_sdk'),
    license='MIT',
    description='Python SDK for Arcgis API',
    long_description=get_long_description(),
    author='mongkok',
    author_email='dani.pyc@gmail.com',
    maintainer='mongkok',
    url='https://github.com/mongkok/arcgis-sdk/',
    packages=find_packages(exclude=['tests*']),
    install_requires=[
        'requests>=2.14.2'
    ],
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],
    zip_safe=False,
    setup_requires=[
        'pytest-runner'
    ],
    tests_require=[
        'pytest>=3.0.7',
        'requests>=2.14.2',
        'responses>=0.5.1'
    ]
)
